/*
marquee.c Version 0.1.0. ISO8859 Marquee in X Windows.
Copyright (C) 2001  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* to define the escape key */
#define XK_MISCELLANY 1

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/keysymdef.h>
#include <assert.h>
#include <unistd.h>
#include "12x24.h"

#define FONT "6x13"
#define NIL (0)
#define SCRW 624
#define SCRH 400
#define CIRCLE 23040
#define SZ 1

int whiteColor,blackColor;
int runflg,expflg;
unsigned long yellow;
Display *dpy;
Window w;
GC gc;
Font fontid;
Screen *dfltscr;
Colormap cmap;
XColor scrdef,exctdef;

/* X Windows code is based on: */
/* http://tronche.lri.fr:8000/gui/x/xlib-tutorial/2nd-program-anatomy.html */

void getkey()
   {
   int i,rslt,msk;
   int symbol;
   int XCheckMaskEvent();
   XEvent e;
   XKeyEvent *k;
   char spc[32];

   msk = KeyPressMask|ExposureMask;

   XSelectInput(dpy, w, msk);

   while (XCheckMaskEvent(dpy, msk, &e))
      {
      if (e.type == KeyPress)
         {
         k = (XKeyEvent *) &e;
         symbol = XLookupKeysym(k,0);
         if (symbol == XK_Escape)
            {
            runflg = 0;
            } /* if quit */
         } /* if keypress event */
      else if (e.type == Expose)
         {
         expflg = 1;
         } /* if expose event */
      } /* if event received */
   } /* getkey */

void initx()
   {
   int i,rslt;
   char title[64];

   dpy = XOpenDisplay(NIL);

   if (dpy == NULL)
      {
      fprintf(stderr,"X Windows failure\n");
      exit(1);
      } /* if X Windows is not active */

   assert(dpy);

   whiteColor = WhitePixel(dpy, DefaultScreen(dpy));
   blackColor = BlackPixel(dpy, DefaultScreen(dpy));

   w = XCreateSimpleWindow(dpy,
      DefaultRootWindow(dpy),
      0, 0, 
      SCRW, SCRH,
      0, whiteColor,
      whiteColor);

   XSelectInput(dpy, w, StructureNotifyMask);

   XMapWindow(dpy, w);

   gc = XCreateGC(dpy, w, 0, NIL);

   fontid = (Font) XLoadFont(dpy,FONT);

   XSetFont(dpy,gc,fontid);

   XSetForeground(dpy, gc, blackColor);

   dfltscr = XDefaultScreenOfDisplay(dpy);
   if (dfltscr == NULL)
      {
      fprintf(stderr,"XDefaultScreenOfDisplay failed\n");
      perror("XDefaultScreenOfDisplay failed");
      exit(1);
      } /* if error */

   cmap = XDefaultColormapOfScreen(dfltscr);

   rslt = XAllocNamedColor(dpy,cmap,"yellow",
      &scrdef,&exctdef);
   if (rslt < 0)
      {
      fprintf(stderr,"XAllocNamedColor failed\n");
      perror("XAllocNamedColor failed");
      exit(1);
      } /* if error */
   yellow = scrdef.pixel;

   XSetWindowBorderWidth(dpy, w, 40);

   sprintf(title,"marquee");
   XStoreName(dpy,w,title);

   XMoveWindow(dpy,w,0,0);

   while(1)
      {
      XEvent e;
      XNextEvent(dpy, &e);
      if (e.type == MapNotify) break;
      } /* wait for window initialization */

   XSetForeground(dpy, gc, blackColor);

   XDrawRectangle(dpy,w,gc,0,100,SCRW,150);

   XFillRectangle(dpy,w,gc,0,100,SCRW,150);

   } /* initx */

int getbyte()
   {
   int len;
   unsigned char buf[8];
   unsigned char *p;
   p = buf;
   len = read(0,buf,1);
   if (len == 0)
      return(EOF);
   else if (len < 0)
      {
      fprintf(stderr,"Error reading stdin\n");
      perror("Read error");
      exit(1);
      } /* if read err */
   else
      return(*p);
   } /* getbyte */

void putmq(ch,x,y)
unsigned char ch;
int x,y;
   {
   int i,j,col,row;
   char *p;
   p = (char *) font + (ch * 12 * 24);
   for (j=0;j<24;j++)
      {
      for (i=0;i<12;i++)
         {
         if (*p == 1)
            {
            col = x + (i * 4);
            row = y + (j * 4);
            XDrawArc(dpy,w,gc,col,row,SZ,SZ,0,CIRCLE);
            XFillArc(dpy,w,gc,col,row,SZ,SZ,0,CIRCLE);
            }
         p++;
         } /* for each col */
      } /* for each row */
   } /* putmq */

void putmsg(str)
unsigned char *str;
   {
   int i,col;
   unsigned char *p,*q;
   p = str;
   while (runflg)
      {
      col = 10;
      XSetForeground(dpy, gc, blackColor);
      XDrawRectangle(dpy,w,gc,0,100,SCRW,150);
      XFillRectangle(dpy,w,gc,0,100,SCRW,150);
      XSetForeground(dpy, gc, yellow);
      for (i=0;i<12;i++)
         {
         q = p+i;
         if (*q == '\0')
            {
            p = str - 1;
            break;
            } /* if end of string */
         putmq(*q,col,125);
         col += 50;
         }
      p++;
      if (*p == '\0') p = str;
      p++;
      if (*p == '\0') p = str;
      getkey();
      if (expflg)
         {
         expflg = 0;
         XSetForeground(dpy, gc, blackColor);
         XDrawRectangle(dpy,w,gc,0,100,SCRW,150);
         XFillRectangle(dpy,w,gc,0,100,SCRW,150);
         XSetForeground(dpy, gc, yellow);
         } /* if exposed */
      sleep(1);
      } /* msg loop */
   } /* putmsg */

int main()
   {
   int ch;
   unsigned char *str;
   unsigned char *p,*q;
   str = (char *) malloc(1024*1024);
   if (str == NULL)
      {
      fprintf(stderr,"Not enough memory.\n");
      exit(1);
      } /* not enough memory */
   initx();
   runflg = 1;
   expflg = 0;
   strcpy(str,"            ");
   p = str+12;
   q = str + (1024*1024) - 64;
   while (p < q)
      {
      ch = getbyte();
      if (ch == EOF) break;
      else if (ch < ' ') ch = ' ';
      else if (ch == 127) ch = ' ';
      *p++ = (unsigned char) ch;
      } /* fill str */
   *p = '\0';
   if (strlen(str) == 12) return(0);
   strcat(str,"            ");
   putmsg(str);
   return(0);
   } /* main */
